import {
  select,
  json,
  zoom,
  drag,
  event,
  tree,
  hierarchy,
} from 'd3';

export function renderPromoted(data, options = {}) {
  const {
    duration = 350,
    radius = 6, // radius of curve for links
    barHeight = 40,
    margin = {
      top: 100,
      left: 20,
      bottom: 50,
      right: 30,
    },
    index_width = 400, // this svg
    working_width = 800, // next door svg
    svg_height = 800,
    svg_spacing = 500,
    // Icons
    prefix = 'https://raw.githubusercontent.com/os-threat/images/main/img/',
    shape = 'rect-',
    icon_sz = 60,
    textPadding = 8,
    corner = 5,
    // the tree view
    minHeight = 20,
    width = working_width,
    height = svg_height,
    lineSpacing = 60,
    lineThickness = 1,
    indentSpacing = 50,
    itemFont = '18px',
    boxSize = 10,
  } = options;

  console.log('Inside promoted', data);

  const svg = d3
    .select('#promoted_svg')
    .append('svg')
    .attr('width', working_width)
    .attr('height', svg_height / 3)
    .attr('class', 'promoted_svg')
    // .attr('transform', 'translate(' + 20 + ',' + 50 + ')')
    .attr('viewBox', [0, 0, working_width, svg_height / 3])
    .call(
      d3.zoom().on('zoom', function () {
        svg.attr('transform', d3.event.transform);
      }),
    )
    .on('dblclick.zoom', null)
    .append('g');

  const trans_left = -0;
  const trans_top = -20;

  const rect_svg = svg
    .append('rect')
    .attr('class', 'promoted_rect')
    .attr('width', working_width)
    .attr('height', 266)
    .attr('stroke', 'grey')
    .attr('fill', 'white')
    .attr(
      'transform',
      'translate(' + trans_left + ',' + trans_top + ')',
    );

  const nodes = data.nodes;
  const edges = data.edges;
  // setup group for all graph objects
  const g = svg.append('g').attr('class', 'graph');

  // setup the simulation forces
  var simulation = d3
    .forceSimulation()
    .force(
      'link',
      d3
        .forceLink()
        .id(function (d) {
          return d.id;
        })
        .distance(100),
    )
    .force('charge', d3.forceManyBody(-200))
    .force(
      'center',
      d3.forceCenter(working_width / 2, svg_height / 6),
    );
  // Setup Links
  const edge = g
    .append('g')
    .attr('class', 'edges')
    .selectAll('line')
    .data(edges)
    .enter()
    .append('line')
    .attr('stroke-width', '1px')
    .attr('stroke', 'black');

  // Setup Nodes Without Shape First
  const node = g
    .append('g')
    .attr('class', 'nodes')
    .selectAll('g')
    .data(nodes)
    .enter()
    .append('image')
    .attr('xlink:href', function (d) {
      return prefix + shape + d.icon + '.svg';
    })
    .attr('x', (d) => {
      d.x - icon_sz / 2;
    })
    .attr('y', (d) => {
      d.y - icon_sz / 2;
    })
    .attr('width', icon_sz)
    .attr('height', icon_sz)
    .attr('class', (d) => d.type)
    .attr('id', (d) => d.id);

  // Setup Shapes Variable, so it draws before text

  // setup the text, headings, without descriptions, to start for each node
  var headings = node
    .append('text')
    .text((d) => d.type)
    .attr('id', (d) => 'heading-' + d.id)
    .style('font-size', 10)
    .style('text-anchor', 'middle')
    .style('color', 'black')
    .style('fill', 'black')
    .style('pointer-events', 'none');

  // Simulation Run Time
  simulation.nodes(nodes).on('tick', ticked);

  simulation.force('link').links(edges);

  function ticked() {
    edge
      .attr('x1', function (d) {
        return d.source.x;
      })
      .attr('y1', function (d) {
        return d.source.y;
      })
      .attr('x2', function (d) {
        return d.target.x;
      })
      .attr('y2', function (d) {
        return d.target.y;
      });

    node.attr('transform', function (d) {
      return 'translate(' + d.x + ',' + d.y + ')';
    });
  }

  // Setup the svg zooming

  node.on('dblclick', releasenode);

  var drag_handler = d3
    .drag()
    .on('start', drag_start)
    .on('drag', drag_drag)
    .on('end', drag_end);

  drag_handler(node);

  //Drag functions
  //d is the node
  function drag_start(d) {
    if (!d3.event.active)
      simulation.alphaTarget(0.3).restart();
    d.fx = d.x;
    d.fy = d.y;
  }

  //make sure you can't drag the circle outside the box
  function drag_drag(d) {
    d.fx = d3.event.x;
    d.fy = d3.event.y;
  }

  function drag_end(d) {
    if (!d3.event.active) simulation.alphaTarget(0.3);
    d.fx = d.x;
    d.fy = d.y;
  }
  function releasenode(d) {
    console.log(
      '================= release node ==========',
    );
    console.log('d.fx  ', d.fx, ' d.fy ', d.fy);
    console.log('d.x  ', d.x, ' d.y ', d.y);
    d3.select(this).classed('fixed', (d.fixed = false));
    d.fx = null;
    d.fy = null;
    if (!d3.event.active) simulation.alphaTarget(0.9);
  }
}
