import {
  select,
  json,
  zoom,
  drag,
  event,
  tree,
  hierarchy,
} from 'd3';

function makeLink(start, end, radius) {
  const path = d3.path();
  const dh = (4 / 3) * Math.tan(Math.PI / 8); // tangent handle offset

  //flip curve
  let fx, fy;
  if (end[0] - start[0] == 0) {
    fx = 0;
  } else if (end[0] - start[0] > 0) {
    fx = 1;
  } else {
    fx = -1;
  }
  if (end[1] - start[1] == 0) {
    fy = 0;
  } else if (end[1] - start[1] > 0) {
    fy = 1;
  } else {
    fy = -1;
  }

  //scale curve when dx or dy is less than the radius
  if (radius == 0) {
    fx = 0;
    fy = 0;
  } else {
    fx *=
      Math.min(Math.abs(start[0] - end[0]), radius) /
      radius;
    fy *=
      Math.min(Math.abs(start[1] - end[1]), radius) /
      radius;
  }

  path.moveTo(...start);
  path.lineTo(...[start[0], end[1] - fy * radius]);
  path.bezierCurveTo(
    ...[start[0], end[1] + fy * radius * (dh - 1)],
    ...[start[0] + fx * radius * (1 - dh), end[1]],
    ...[start[0] + fx * radius, end[1]],
  );
  path.lineTo(...end);
  return path;
}

export function renderTree(
  data,
  interior_svg,
  options = {},
) {
  const {
    duration = 350,
    radius = 6, // radius of curve for links
    barHeight = 40,
    margin = {
      top: 100,
      left: 20,
      bottom: 50,
      right: 30,
    },
    index_width = 400, // this svg
    working_width = 800, // next door svg
    svg_height = 800,
    svg_spacing = 500,
    // Icons
    prefix = 'https://raw.githubusercontent.com/os-threat/images/main/img/',
    shape = 'rect-',
    icon_size = 36,
    textPadding = 8,
    corner = 5,
    // the tree view
    minHeight = 20,
    width = index_width,
    height = svg_height,
    lineSpacing = 60,
    lineThickness = 1,
    indentSpacing = 50,
    itemFont = '18px',
    boxSize = 10,
  } = options;
  // WARNING: x and y are switched because the d3.tree is vertical rather than the default horizontal
  // settings

  let plus = {
    shapeFill: 'Gray',
    shapeStroke: 'Gray',
    textFill: 'white',
    text: '+',
  };
  let minus = {
    shapeFill: 'white',
    shapeStroke: 'Gray',
    textFill: 'Gray',
    text: '−',
  };

  console.log('svg->', tree_svg);

  const trans_left = -margin.left;
  const trans_top = -margin.top;

  var tree_svg = interior_svg.attr('class', 'index_svg');

  var tree_rect = tree_svg
    .append('rect')
    .attr('class', 'index_rect')
    .attr('width', index_width)
    .attr('height', svg_height)
    .attr('stroke', 'grey')
    .attr('fill', 'white')
    .attr(
      'transform',
      'translate(' + trans_left + ',' + trans_top + ')',
    );

  //
  let tree = d3
    .tree()
    .nodeSize([lineSpacing, indentSpacing]);

  let root = d3.hierarchy(data);

  root.x0 = 0;
  root.y0 = 0;
  root.descendants().forEach((d, i) => {
    d.id = i;
    d._children = d.children;
    if (d.depth && d.data.name.length !== 7)
      d.children = null;
  });
  console.log('In and initialised');

  let index = -1;
  root.eachBefore(function (n) {
    ++index;
  }); // counts original number of items

  //resonsive size before icons had height =  Math.max(minHeight, index * lineSpacing + marginTop + marginBottom )
  const svg = tree_svg
    //.attr("viewBox", [-marginLeft, -marginTop, width, height])
    .attr('width', width + margin.left + margin.right)
    .attr('height', height + margin.top + margin.bottom)
    .style('font', '10px sans-serif');

  const gLink = svg
    .append('g')
    .attr('fill', 'none')
    .attr('stroke', '#AAA')
    .attr('stroke-width', 0.75);

  const gNode = svg
    .append('g')
    .attr('cursor', 'pointer')
    .attr('pointer-events', 'all');

  console.log('Ready to update');

  let indexLast;
  function update(source) {
    const nodes = root.descendants().reverse();
    const links = root.links();

    // Compute the new tree layout.
    tree(root);

    // node position function
    index = -1;
    root.eachBefore(function (n) {
      n.x = ++index * lineSpacing;
      n.y = n.depth * indentSpacing;
    });

    const height = Math.max(
      minHeight,
      index * lineSpacing + margin.top + margin.bottom,
    );

    const transition = svg
      .transition()
      .duration(duration)
      .attr('height', height)
      .attr('viewBox', [
        -margin.left,
        -margin.top,
        width,
        height,
      ])
      .tween(
        'resize',
        window.ResizeObserver
          ? null
          : () => () => svg.dispatch('toggle'),
      );

    console.log('PAst Transition');

    // Update the nodes…
    const node = gNode
      .selectAll('g')
      .data(nodes, (d) => d.id);

    // Enter any new nodes at the parent's previous position.
    const nodeEnter = node
      .enter()
      .append('g')
      .attr(
        'transform',
        (d) => `translate(${d.y},${source.x0})`,
      )
      .attr('fill-opacity', 0)
      .attr('stroke-opacity', 0)
      .on('click', (d) => {
        d.children = d.children ? null : d._children;
        update(d);

        charge
          .attr('fill', (d) =>
            d._children
              ? d.children
                ? minus.textFill
                : plus.textFill
              : 'none',
          )
          .text((d) =>
            d._children
              ? d.children
                ? minus.text
                : plus.text
              : '',
          );

        box.attr('fill', (d) =>
          d._children
            ? d.children
              ? minus.shapeFill
              : plus.shapeFill
            : 'none',
        );
      });

    // check box
    let box = nodeEnter
      .append('rect')
      .attr('width', boxSize)
      .attr('height', boxSize)
      .attr('x', -boxSize / 2)
      .attr('y', -boxSize / 2)
      .attr('fill', (d) =>
        d._children
          ? d.children
            ? minus.shapeFill
            : plus.shapeFill
          : 'none',
      )
      .attr('stroke', (d) =>
        d._children ? 'black' : 'none',
      )
      .attr('stroke-width', 0.5);

    // check box symbol
    let charge = nodeEnter
      .append('text')
      .attr('x', 0)
      .attr('text-anchor', 'middle')
      .attr('alignment-baseline', 'central')
      .attr('fill', (d) =>
        d._children
          ? d.children
            ? minus.textFill
            : plus.textFill
          : 'none',
      )
      .text((d) =>
        d._children ? (d.children ? '−' : '+') : '',
      );

    console.log('Ready to Setup icons');

    // attach icon
    let image = nodeEnter
      .append('image')
      .attr('x', 10 + boxSize)
      .attr('y', -3 - icon_size / 2)
      .attr('dy', -icon_size)
      .attr('xlink:href', function (d) {
        //console.log(prefix + shape + d.data.icon + '.svg');
        return prefix + shape + d.data.icon + '.svg';
      })
      .attr('width', function (d) {
        return icon_size + 5;
      })
      .attr('height', function (d) {
        return icon_size + 5;
      });

    // label text
    let label = nodeEnter
      .append('text')
      .attr('x', icon_size + 30 + boxSize / 2)
      .attr('text-anchor', 'start')
      .style('font-size', itemFont)
      .attr('dy', '0.32em')
      .text((d) => d.data.type);

    // Transition nodes to their new position.
    const nodeUpdate = node
      .merge(nodeEnter)
      .transition(transition)
      .attr('transform', (d) => `translate(${d.y},${d.x})`)
      .attr('fill-opacity', 1)
      .attr('stroke-opacity', 1);

    // Transition exiting nodes to the parent's new position.
    const nodeExit = node
      .exit()
      .transition(transition)
      .remove()
      .attr(
        'transform',
        (d) => `translate(${d.y},${source.x})`,
      )
      .attr('fill-opacity', 0)
      .attr('stroke-opacity', 0);

    // Update the links…
    const link = gLink
      .selectAll('path')
      .data(links, (d) => d.target.id);

    // Enter any new links at the parent's previous position.
    const linkEnter = link
      .enter()
      .append('path')
      .attr('stroke-opacity', 0)
      .attr('d', (d) =>
        makeLink(
          [d.source.y, source.x],
          [
            d.target.y +
              (d.target._children ? 0 : boxSize / 2),
            source.x,
          ],
          radius,
        ),
      );

    // Transition links to their new position.
    link
      .merge(linkEnter)
      .transition(transition)
      .attr('stroke-opacity', 1)
      .attr('d', (d) =>
        makeLink(
          [d.source.y, d.source.x],
          [
            d.target.y +
              (d.target._children ? 0 : boxSize / 2),
            d.target.x,
          ],
          radius,
        ),
      );

    // Transition exiting nodes to the parent's new position.
    link
      .exit()
      .transition(transition)
      .attr('stroke-opacity', 0)
      .attr('d', (d) =>
        makeLink(
          [d.source.y, source.x],
          [
            d.target.y +
              (d.target._children ? 0 : boxSize / 2),
            source.x,
          ],
          radius,
        ),
      );

    // Stash the old positions for transition.
    root.eachBefore((d) => {
      d.x0 = d.x;
      d.y0 = d.y;
    });

    indexLast = index; // to know if viewbox is expanding or contracting
  }

  update(root);

  return svg.no